@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2011 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,6 +20,11 @@
 #define PHP_SPL_H
 
 #include "php.h"
+#if defined(PHP_WIN32)
+# include "win32/php_stdint.h"
+#elif defined(HAVE_STDINT_H)
+# include <stdint.h>
+#endif
 #include <stdarg.h>
 
 #if 0
@@ -32,15 +37,17 @@ extern zend_module_entry spl_module_entry;
 #define phpext_spl_ptr &spl_module_entry
 
 #ifdef PHP_WIN32
-# ifdef SPL_EXPORTS
-#  define SPL_API __declspec(dllexport)
-# elif defined(COMPILE_DL_SPL)
-#  define SPL_API __declspec(dllimport)
-# else
-#  define SPL_API /* nothing */
-# endif
+#	ifdef SPL_EXPORTS
+#		define SPL_API __declspec(dllexport)
+#	elif defined(COMPILE_DL_SPL)
+#		define SPL_API __declspec(dllimport)
+#	else
+#		define SPL_API /* nothing */
+#	endif
+#elif defined(__GNUC__) && __GNUC__ >= 4
+#	define SPL_API __attribute__ ((visibility("default")))
 #else
-# define SPL_API
+#	define SPL_API
 #endif
 
 #if defined(PHP_WIN32) && !defined(COMPILE_DL_SPL)
@@ -60,6 +67,9 @@ ZEND_BEGIN_MODULE_GLOBALS(spl)
 	HashTable *  autoload_functions;
 	int          autoload_running;
 	int          autoload_extensions_len;
+	zend_intptr_t     hash_mask_handle;
+	zend_intptr_t     hash_mask_handlers;
+	int          hash_mask_init;
 ZEND_END_MODULE_GLOBALS(spl)
 
 #ifdef ZTS
@@ -74,6 +84,8 @@ PHP_FUNCTION(spl_classes);
 PHP_FUNCTION(class_parents);
 PHP_FUNCTION(class_implements);
 
+PHPAPI void php_spl_object_hash(zval *obj, char* md5str TSRMLS_DC);
+
 #endif /* PHP_SPL_H */
 
 /*
