@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2011 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php.h 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php.h 306939 2011-01-01 02:19:59Z felipe $ */
 
 #ifndef PHP_H
 #define PHP_H
@@ -26,7 +26,7 @@
 #include <dmalloc.h>
 #endif
 
-#define PHP_API_VERSION 20041225
+#define PHP_API_VERSION 20090626
 #define PHP_HAVE_STREAMS
 #define YYDEBUG 0
 
@@ -40,29 +40,27 @@
 #undef sprintf
 #define sprintf php_sprintf
 
-#if SUHOSIN_PATCH
-#if HAVE_REALPATH
-#undef realpath
-#define realpath php_realpath
-#endif
-#endif
-
 /* PHP's DEBUG value must match Zend's ZEND_DEBUG value */
 #undef PHP_DEBUG
 #define PHP_DEBUG ZEND_DEBUG
 
 #ifdef PHP_WIN32
-#include "tsrm_win32.h"
-#include "win95nt.h"
+#	include "tsrm_win32.h"
+#	include "win95nt.h"
 #	ifdef PHP_EXPORTS
-#	define PHPAPI __declspec(dllexport)
+#		define PHPAPI __declspec(dllexport)
 #	else
-#	define PHPAPI __declspec(dllimport)
+#		define PHPAPI __declspec(dllimport)
 #	endif
-#define PHP_DIR_SEPARATOR '\\'
-#define PHP_EOL "\r\n"
+#	define PHP_DIR_SEPARATOR '\\'
+#	define PHP_EOL "\r\n"
 #else
-#define PHPAPI
+#	if defined(__GNUC__) && __GNUC__ >= 4
+#		define PHPAPI __attribute__ ((visibility("default")))
+#	else
+#		define PHPAPI
+#	endif
+
 #define THREAD_LS
 #define PHP_DIR_SEPARATOR '/'
 #if defined(__MacOSX__)
@@ -78,8 +76,6 @@
 #define PHP_OS      PHP_UNAME
 #endif
 
-#include "php_regex.h"
-
 #if HAVE_ASSERT_H
 #if PHP_DEBUG
 #undef NDEBUG
@@ -177,6 +173,13 @@ typedef unsigned int socklen_t;
 # endif
 #endif
 
+#ifndef va_copy
+# ifdef __va_copy
+#  define va_copy(ap1, ap2)         __va_copy((ap1), (ap2))
+# else
+#  define va_copy(ap1, ap2)         memcpy((&ap1), (&ap2), sizeof(va_list))
+# endif
+#endif
 
 #include "zend_hash.h"
 #include "php3_compat.h"
@@ -200,10 +203,6 @@ typedef unsigned int socklen_t;
 char *strerror(int);
 #endif
 
-#if (REGEX == 1 || REGEX == 0) && !defined(NO_REGEX_EXTRA_H)
-#include "regex/regex_extra.h"
-#endif
-
 #if HAVE_PWD_H
 # ifdef PHP_WIN32
 #include "win32/param.h"
@@ -286,16 +285,14 @@ int cfgparse(void);
 END_EXTERN_C()
 
 #define php_error zend_error
-
-typedef enum {
-	EH_NORMAL = 0,
-	EH_SUPPRESS,
-	EH_THROW
-} error_handling_t;
+#define error_handling_t zend_error_handling_t
 
 BEGIN_EXTERN_C()
-PHPAPI void php_set_error_handling(error_handling_t error_handling, zend_class_entry *exception_class TSRMLS_DC);
-#define php_std_error_handling() php_set_error_handling(EH_NORMAL, NULL TSRMLS_CC)
+static inline ZEND_ATTRIBUTE_DEPRECATED void php_set_error_handling(error_handling_t error_handling, zend_class_entry *exception_class TSRMLS_DC)
+{
+	zend_replace_error_handling(error_handling, exception_class, NULL TSRMLS_CC);
+}
+static inline ZEND_ATTRIBUTE_DEPRECATED void php_std_error_handling() {}
 
 PHPAPI void php_verror(const char *docref, const char *params, int type, const char *format, va_list args TSRMLS_DC) PHP_ATTRIBUTE_FORMAT(printf, 4, 0);
 
@@ -312,6 +309,9 @@ PHPAPI void php_error_docref1(const char *docref TSRMLS_DC, const char *param1,
 	PHP_ATTRIBUTE_FORMAT(printf, PHP_ATTR_FMT_OFFSET + 4, PHP_ATTR_FMT_OFFSET + 5);
 PHPAPI void php_error_docref2(const char *docref TSRMLS_DC, const char *param1, const char *param2, int type, const char *format, ...)
 	PHP_ATTRIBUTE_FORMAT(printf, PHP_ATTR_FMT_OFFSET + 5, PHP_ATTR_FMT_OFFSET + 6);
+#ifdef PHP_WIN32
+PHPAPI void php_win32_docref2_from_error(DWORD error, const char *param1, const char *param2 TSRMLS_DC);
+#endif
 END_EXTERN_C()
 
 #define php_error_docref php_error_docref0
@@ -327,12 +327,10 @@ END_EXTERN_C()
 
 /* functions */
 BEGIN_EXTERN_C()
-int php_register_internal_extensions(TSRMLS_D);
-
-int php_mergesort(void *base, size_t nmemb, register size_t size, int (*cmp)(const void *, const void * TSRMLS_DC) TSRMLS_DC);
-
+PHPAPI extern int (*php_register_internal_extensions_func)(TSRMLS_D);
+PHPAPI int php_register_internal_extensions(TSRMLS_D);
+PHPAPI int php_mergesort(void *base, size_t nmemb, register size_t size, int (*cmp)(const void *, const void * TSRMLS_DC) TSRMLS_DC);
 PHPAPI void php_register_pre_request_shutdown(void (*func)(void *), void *userdata);
-
 PHPAPI void php_com_initialize(TSRMLS_D);
 END_EXTERN_C()
 
